#!/usr/bin/env node
'use strict';

var _open = require('open');

var _open2 = _interopRequireDefault(_open);

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _path = require('path');

var _interpret = require('interpret');

var _interpret2 = _interopRequireDefault(_interpret);

var _lib = require('../lib');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var argv = _yargs2.default.option('client', {
  alias: 'c',
  description: 'Path to client webpack configuration',
  requiresArg: true,
  type: 'string'
}).option('server', {
  alias: 's',
  description: 'Path to server webpack configuration',
  requiresArg: true,
  type: 'string'
}).option('hot', {
  alias: 'H',
  default: true,
  description: 'Enable hot-reload.',
  type: 'boolean'
}).argv;

function registerCompiler(moduleDescriptor) {
  if (moduleDescriptor) {
    if (typeof moduleDescriptor === 'string') {
      require(moduleDescriptor);
    } else if (!Array.isArray(moduleDescriptor)) {
      moduleDescriptor.register(require(moduleDescriptor.module));
    } else {
      for (var i = 0; i < moduleDescriptor.length; i++) {
        try {
          registerCompiler(moduleDescriptor[i]);
          break;
        } catch (e) {
          // do nothing
        }
      }
    }
  }
}

function load(entry) {
  var extensions = Object.keys(_interpret2.default.extensions).sort(function (a, b) {
    return a.length - b.length;
  });
  var configPath = (0, _path.resolve)(entry);
  var ext = undefined;
  for (var i = extensions.length - 1; i >= 0; i--) {
    var tmpExt = extensions[i];
    if (configPath.indexOf(tmpExt, configPath.length - tmpExt.length) > -1) {
      ext = tmpExt;
      break;
    }
  }
  if (!ext) {
    ext = (0, _path.extname)(configPath);
  }
  registerCompiler(_interpret2.default.extensions[ext]);
  var value = require(configPath);
  if (value && value.__esModule) {
    return value.default;
  }
  return value;
}

global.__IN_DEV_SERVER = true;
process.env.HOT = argv.hot;

var server = (0, _lib.createServer)({
  client: load(argv.client),
  server: load(argv.server)
});

/* eslint no-console: 0 */
server.on('ready', function () {
  console.log('💎  Ready.');
});

server.listen(process.env.PORT, function () {
  var url = 'http://localhost:' + server.address().port + '/';
  console.log('💎  Listening: ' + url + '.');
  if (!process.env.PORT) {
    (0, _open2.default)(url);
  }
});
//# sourceMappingURL=webpack-udev-server.js.map