'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createAssetServer;

var _webpackDevMiddleware = require('webpack-dev-middleware');

var _webpackDevMiddleware2 = _interopRequireDefault(_webpackDevMiddleware);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createAssetServer(assets, options) {
  var compiler = (0, _webpack2.default)(assets);
  var middleware = (0, _webpackDevMiddleware2.default)(compiler, _extends({}, options, {
    publicPath: assets.output.publicPath
  }));
  var server = _http2.default.createServer(function (req, res) {
    middleware(req, res, function () {
      res.writeHead(404);
      res.end();
    });
  });
  var io = (0, _socket2.default)(server, {
    path: assets.output.publicPath + '/socket.io'
  });

  var stats = null;

  function ok(stats) {
    return (!stats.errors || stats.errors.length === 0) && stats.assets && stats.assets.every(function (asset) {
      return !asset.emitted;
    });
  }

  function sendStats(socket, force) {
    if (!stats) {
      return;
    } else if (!force && ok(stats)) {
      socket.emit('still-ok');
      return;
    }

    socket.emit('hash', stats.hash);
    if (stats.errors && stats.errors.length > 0) {
      socket.emit('errors', stats.errors);
    } else if (stats.warnings && stats.warnings.length > 0) {
      socket.emit('warnings', stats.warnings);
    } else {
      socket.emit('ok');
    }
  }

  // Socket events.
  io.sockets.on('connection', function (socket) {
    socket.emit('hot');
    sendStats(socket, true);
  });

  // Compiler events.
  compiler.plugin('compile', function () {
    return io.sockets.emit('invalid');
  });
  compiler.plugin('invalid', function () {
    return io.sockets.emit('invalid');
  });
  compiler.plugin('done', function (_stats) {
    stats = _stats;
    sendStats(io.sockets);
  });
  compiler.plugin('done', function (stats) {
    return server.emit('stats', stats);
  });

  return _extends(server, {
    compiler: compiler,
    close: function close() {
      io.close();
      middleware.close();
      _http2.default.Server.prototype.close.apply(this, arguments);
    }
  });
}
//# sourceMappingURL=assets.js.map