'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createRenderServer;

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _child_process = require('child_process');

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _backo = require('backo');

var _backo2 = _interopRequireDefault(_backo);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createRenderServer(renderer, options) {
  var serverCompiler = (0, _webpack2.default)(renderer);
  var backoff = new _backo2.default({ min: 0, max: 1000 * 5 });
  var events = new _events2.default();
  var child = null;
  var addr = null;
  var start = false;
  var rip = false;
  var port = 0;
  var assetStats = null;
  var serverStats = null;
  var hot = true;

  function send() {
    if (child && assetStats) {
      child.send({ type: 'stats', stats: assetStats });
    }
  }

  function kill() {
    if (child) {
      (function () {
        var timeout = null;
        child.once('exit', function () {
          child = null;
          if (timeout) {
            clearTimeout(timeout);
          }
        });
        child.kill('SIGINT');
        timeout = setTimeout(function () {
          child.kill('SIGTERM');
          child = null;
          timeout = null;
        }, 3000);
      })();
    }
  }

  function paths(base, entry) {
    if (Array.isArray(entry)) {
      return entry.map(function (value) {
        return paths(base, value);
      });
    } else if (typeof entry === 'string') {
      return [_path2.default.join(base, entry)];
    }
    throw new TypeError('Invalid module path.');
  }

  function entry(module) {
    var value = module.filter(function (entry) {
      return (/\.js$/.test(entry)
      );
    });
    if (value.length > 0) {
      return value[value.length - 1];
    }
    throw new TypeError('Module has no JavaScript files to run.');
  }

  function _spawn() {
    // `assetsByChunkName` can map to either a string OR an array – since one
    // chunk can have multiple assets (e.g. source maps).
    var map = serverStats.toJson({ assets: true }).assetsByChunkName;
    var modules = Object.keys(map).map(function (key) {
      return paths(renderer.output.path, map[key]);
    });
    var env = _extends({}, process.env, {
      PORT: port,
      HAS_WEBPACK_ASSET_EVENTS: 1,
      HAS_WEBPACK_STATS_EVENTS: 1
    });

    // Only support one entrypoint right now. Maybe support more later.
    if (modules.length !== 1) {
      throw new Error('Must only export 1 entrypoint!');
    }
    var target = entry(modules[0]);
    console.log('▶️  ' + target);
    child = (0, _child_process.fork)(target, [], { env: env });
    child.on('message', function (message) {
      switch (message.type) {
        case 'ping':
          backoff.reset();
          break;
        case 'address':
          addr = message.address;
          events.emit('listening');
          break;
        default:
          events.emit('error', new Error('Unknown event: ' + message.type));
      }
    });
    child.once('exit', function () {
      if (rip) {
        events.emit('close');
      } else {
        spawn();
      }
    });
    child.once('error', function () {
      if (rip) {
        events.emit('close');
      } else {
        spawn();
      }
    });
    send();
  }

  function spawn() {
    setTimeout(_spawn, backoff.duration());
  }

  function reload() {
    if (hot) {
      // If we're already running just invoke HMR, otherwise start up.
      child.kill('SIGUSR2');
    } else {
      kill();
    }
  }

  function trigger() {
    if (!start || !serverStats) {
      return;
    } else if (child) {
      reload();
    } else {
      spawn();
    }
  }

  serverCompiler.watch({}, function (err, _stats) {
    // Bail on failure.
    if (err) {
      return;
    }
    serverStats = _stats;

    /* eslint no-console: 0 */
    console.log(serverStats.toString(options.stats));
    trigger();
  });

  process.once('beforeExit', function () {
    rip = true;
    kill();
  });

  return _extends(events, {
    compiler: serverCompiler,
    close: function close() {
      rip = true;
      kill();
    },
    listen: function listen(_port) {
      start = true;
      port = _port;
      trigger();
    },
    address: function address() {
      return addr;
    },
    assets: function assets() {
      send();
    },
    stats: function stats(_stats) {
      assetStats = _stats.toJson({
        hash: true,
        version: false,
        timings: false,
        assets: true,
        chunks: true,
        chunkModules: false,
        modules: false,
        cached: false,
        reasons: false,
        source: false,
        errorDetails: false,
        chunkOrigins: false
      });
      send();
    }
  });
}
//# sourceMappingURL=renderer.js.map