'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _httpProxy = require('http-proxy');

var _httpProxy2 = _interopRequireDefault(_httpProxy);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _assets = require('./assets');

var _assets2 = _interopRequireDefault(_assets);

var _renderer = require('./renderer');

var _renderer2 = _interopRequireDefault(_renderer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

var options = {
  stats: {
    hash: false,
    cached: false,
    cachedAssets: false,
    colors: true,
    modules: false,
    chunks: false
  }
};

function verify(config) {
  if (!config) {
    return false;
  } else if (Array.isArray(config)) {
    return config.some(verify);
  } else if ((typeof config === 'undefined' ? 'undefined' : _typeof(config)) === 'object') {
    return Object.keys(config).map(function (key) {
      return config[key];
    }).some(verify);
  } else if (typeof config === 'string') {
    return (/webpack-udev-server/.test(config)
    );
  }
  return false;
}

function check(config) {
  if (!verify(config)) {
    throw new TypeError('You must include the `webpack-udev-server` runtime.');
  }
}

var Server = (function (_http$Server) {
  _inherits(Server, _http$Server);

  function Server(_ref) {
    var client = _ref.client;
    var server = _ref.server;

    _classCallCheck(this, Server);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Server).call(this));

    check(client);
    check(server);

    // Set some sane asset path.
    client.output.publicPath = '/_assets';

    // Create the servers.
    _this.assets = (0, _assets2.default)(client, options);
    _this.renderer = (0, _renderer2.default)(server, options);
    _this.proxy = _httpProxy2.default.createProxy({ ws: true });
    _this.ready = false;
    _this.state = {};

    // Inform the renderer where the assets are.
    _this.renderer.assets(client.output.publicPath);

    // Forward normal requests.
    _this.on('request', _this.guard(function (req, res) {
      _this.proxy.web(req, res, {
        target: _this.target('http', req)
      });
    }));

    // Forward web sockets.
    _this.on('upgrade', _this.guard(function (req, socket, head) {
      _this.proxy.ws(req, socket, head, {
        target: _this.target('ws', req)
      });
    }));

    // Handle proxy errors. These can occur if the proxied service crashes
    // the connection dies with it resulting in a socket hang-up.
    _this.proxy.on('error', function (error, req, res) {
      if (!res.headersSent) {
        res.statusCode = 500;
        res.setHeader('Content-Type', 'text/plain');
      }
      res.end('Unable to fetch ' + req.url + '.');
    });
    return _this;
  }

  _createClass(Server, [{
    key: 'listen',
    value: function listen() {
      var _this2 = this;

      // TODO: Handle error events from renderer/assets
      this.renderer.once('listening', function () {
        return _this2.set('renderer', true);
      });
      this.assets.once('listening', function () {
        return _this2.set('assets', true);
      });

      this.assets.on('stats', function (stats) {
        _this2.renderer.stats(stats);
      });

      this.assets.listen(0, 'localhost');
      this.renderer.listen(0, 'localhost');
      _http2.default.Server.prototype.listen.apply(this, arguments);
    }
  }, {
    key: 'close',
    value: function close() {
      this.assets.close();
      this.renderer.close();
      _http2.default.Server.prototype.close.apply(this, arguments);
    }
  }, {
    key: 'target',
    value: function target(proto, req) {
      if (req.url.substring(0, '/_assets'.length) === '/_assets') {
        return proto + '://localhost:' + this.assets.address().port;
      }
      return proto + '://localhost:' + this.renderer.address().port;
    }
  }, {
    key: 'set',
    value: function set(type, value) {
      this.state[type] = value;
      this.ready = this.state.assets && this.state.renderer;
      if (this.ready) {
        this.emit('ready');
      }
    }
  }, {
    key: 'guard',
    value: function guard(fn) {
      var _this3 = this;

      return function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        if (_this3.ready) {
          fn.apply(undefined, args);
        } else {
          _this3.once('ready', function () {
            return fn.apply(undefined, args);
          });
        }
      };
    }
  }]);

  return Server;
})(_http2.default.Server);

exports.default = Server;
//# sourceMappingURL=server.js.map